MENUS = {
    'NAV_MENU': [
        {
            "name": "App1 Feature",
            "url": "/app1-feature"
        }
    ],
    'SECOND_MENU': [
        {
            "name": "Second Menu Feature",
            "url": "named_url"
        }
    ]
}
