MENUS = {
    'NAV_MENU': [
        {
            "name": "App2 Feature",
            "url": "/app2-feature"
        },
        {
            "name": "App2 Second Feature",
            "url": "/app2-second-feature"
        }
    ]
}
